require "net/http"
require "uri"

module BookableCloud
  class ApiClient
    include Thor::Actions

    # class Response
    #   attr_reader :json, :success
    #
    #   def initialize(success: true, json: nil)
    #     @success = success
    #     @json = json
    #   end
    # end

    def initialize(config)
      @config = config
    end

    def tenant_url
      @config["tenant_url"]
    end

    def base_url
      tenant_url.to_s + "/admin"
    end

    def get_theme
      uri = URI.parse(base_url + "/themes/#{@config["theme_id"]}.json")

      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true

      request = Net::HTTP::Get.new(uri.request_uri)
      request["Authorization"] = "Bearer #{@config["access_token"]}"


      http.request(request)
    end

    def list_assets
      json = get_json "/themes/#{@config["theme_id"]}/assets.json"

      if assets = json["assets"]
        assets.collect { |asset| asset["key"] }
      end
    end

    def get_asset(asset)
      json = get_json "/themes/#{@config["theme_id"]}/assets.json", params: { key: asset }

      json["asset"]
    end

    def update_asset(key, body)
      json = put_json "/themes/#{@config["theme_id"]}/assets.json", params: {
        "asset[key]" => key,
        "asset[value]" => body
      }
    end

    private

    def get_json(url, params: nil)
      uri = URI.parse(base_url + url)
      uri.query = URI.encode_www_form(params) if params

      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true

      request = Net::HTTP::Get.new(uri.request_uri)
      request["Authorization"] = "Bearer #{@config["access_token"]}"

      response = http.request(request)

      JSON.parse(response.body)
    end

    def put_json(url, params: nil)
      uri = URI.parse(base_url + url)

      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true

      request = Net::HTTP::Put.new(uri.request_uri)
      request.set_form_data(params) if params
      request["Authorization"] = "Bearer #{@config["access_token"]}"

      response = http.request(request)

      # p response.body
      JSON.parse(response.body)
    end
  end
end
